"""Cryptographic functions to sign and verify signatures with STARK keys.

The code in this module is copied from
    https://github.com/starkware-libs/starkex-resources
and is used in accordance with the Apache License, Version 2.0.

https://www.starkware.co/open-source-license/
"""
